﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectAirFighter.DrawningObjects;
using ProjectAirFighter.MovementStrategy;

namespace ProjectAirFighter
{
    public partial class FormAirFighter : Form
    {

        private DrawningAirplane? _drawningAirplane;

        private AbstractStrategy? _abstractStrategy;

        public DrawningAirplane? SelectedAirplane { get; private set; }
        public FormAirFighter()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedAirplane = null;
        }

        private void Draw()
        {
            if (_drawningAirplane == null)
                return;

            Bitmap bmp = new(pictureBoxAirFighter.Width, pictureBoxAirFighter.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningAirplane.DrawTransport(gr);
            pictureBoxAirFighter.Image = bmp;
        }

        private void ButtonCreateAirplane_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));

            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                bodyColor = dialog.Color;
            }
            _drawningAirplane = new DrawningAirplane(random.Next(100, 300), random.Next(1000, 3000),
                bodyColor,pictureBoxAirFighter.Width, pictureBoxAirFighter.Height);
            _drawningAirplane.SetPosition(random.Next(10, 100), random.Next(70, 100));
            Draw();
        }

        private void ButtonMoveClick(object sender, EventArgs e)
        {
            if (_drawningAirplane == null)
                return;

            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningAirplane.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningAirplane.MoveTransport(DirectionType.Down);
                    break;
                case "buttonRight":
                    _drawningAirplane.MoveTransport(DirectionType.Right);
                    break;
                case "buttonLeft":
                    _drawningAirplane.MoveTransport(DirectionType.Left);
                    break;
            }
            Draw();
        }

        private void ButtonCreateAirFighter_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
                bodyColor = dialog.Color;
            if (dialog.ShowDialog() == DialogResult.OK)
                additionalColor = dialog.Color;

            _drawningAirplane = new DrawningAirFighter(random.Next(100, 300), random.Next(1000, 3000),
                bodyColor,additionalColor, Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)), pictureBoxAirFighter.Width, pictureBoxAirFighter.Height);
            _drawningAirplane.SetPosition(random.Next(10, 100), random.Next(70, 100));
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawningAirplane == null)
                return;
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new
                DrawningObjectAirplane(_drawningAirplane), pictureBoxAirFighter.Width,
                pictureBoxAirFighter.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }

        private void buttonSelectAirplane_Click(object sender, EventArgs e)
        {
            SelectedAirplane = _drawningAirplane;
            DialogResult = DialogResult.OK;
        }
    }
}
